//-------------------------------------------------------
// Dier Code wurde aus dem Mikrocontroller.net entnommen
// Author: P. Diener
//
// und entsprechend modifiziert.
// ISR (USART1_RX_vect) setzt das Flag INT_USART
// TODO: Die Funktionen mssen noch auf die USART1 umbenannt werden
//
//





#include <avr/io.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "../HomeReceiver/global.h"


#define FOSC 16000000


#define Tx0BufSize 256
#define Rx0BufSize 256
volatile unsigned char uart0TxBuf[Tx0BufSize], uart0RxBuf[Rx0BufSize], Tx0Empty = 1;
volatile unsigned int uart0TxInBufPtr = 0, uart0RxInBufPtr = 0;
volatile unsigned int uart0TxOutBufPtr = 0, uart0RxOutBufPtr = 0;
volatile unsigned char Rx0Overflow = 0;

char printBuffer[256];
uint8_t hex[16]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

extern volatile


void USART0_Init( unsigned long BAUD )
{
  //crystal oscillator @ 7.3728 MHz
  // #define FOSC 7372800
  unsigned int ubrr = (unsigned int) (FOSC/16uL/ (unsigned long) (BAUD)) - 1;    //example: ubrr should be 47 at 9600 baud!
    // Set the baud rate registers
    UBRR1H = (unsigned char) (ubrr >> 8);
    UBRR1L = (unsigned char) ubrr;
    // Enable receiver and transmitter and enable receive and transmit complete interrupts
    UCSR1B = ((1<<RXCIE1)|/*(1<<TXCIE0)|*/(1<<RXEN1)|(1<<TXEN1));
    // Set frame format: 8data, 1stop bit
    UCSR1C = ((0<<USBS1)|(3<<UCSZ10));
}


//usart0 receive complete interrupt
ISR (USART1_RX_vect)
{
  char dummy;
  unsigned int Ptr;

  INT_USART = 1;


  Ptr = uart0RxInBufPtr + 1;
  if (Ptr >= Rx0BufSize) Ptr = 0;
  if (Ptr != uart0RxOutBufPtr)
  {
    uart0RxBuf[Ptr] = UDR1;
    uart0RxInBufPtr = Ptr;
  }
  else        //no more space in ReceiveBuffer, set overflow warning!
  {
    dummy = UDR1;
    Rx0Overflow = 1;
  }
}



signed int USART0_getchar(void)
{
  if (uart0RxInBufPtr != uart0RxOutBufPtr)
  {
    uart0RxOutBufPtr++;
    if (uart0RxOutBufPtr >= Rx0BufSize) uart0RxOutBufPtr = 0;
    return (signed int) uart0RxBuf[uart0RxOutBufPtr];
  }
  else return -1;        //sorry, there was nothing to read
}


//usart0 transmit data register empty interrupt
ISR (USART1_UDRE_vect)
{




  if (uart0TxInBufPtr != uart0TxOutBufPtr)
  {
    uart0TxOutBufPtr++;
    if (uart0TxOutBufPtr >= Tx0BufSize) uart0TxOutBufPtr = 0;
    UDR1 = uart0TxBuf[uart0TxOutBufPtr];
  }
  else
  {
    Tx0Empty = 1;
    UCSR1B &= ~(1<<UDRIE1);
    //asm volatile ("cbi 154, 5");    //clear bit UDRIE0 in register UCSR0B
  }

}



signed int USART0_putchar(char ch)
{
  unsigned int Ptr;

  Ptr = uart0TxInBufPtr + 1;
  if (Ptr >= Tx0BufSize) Ptr = 0;
  if (Ptr != uart0TxOutBufPtr)
  {
    uart0TxBuf[Ptr] = ch;
    uart0TxInBufPtr = Ptr;
    if (Tx0Empty == 1)
    {
      UCSR1A |= (1<<UDRE1);    //clear data register empty flag - so interrupt will fire when UDRIE0 is set
      UCSR1B |= (1<<UDRIE1);    //Enable data register empty interrupt - force interrupt trigger
    }

    Tx0Empty = 0;
      return 0;
  }
  else
  {
    return -1;      //no room - queue is full
  }
}



void USART0_Puts (char *s)
{
    while (*s)
    {   // as long as *s != '\0' which is the "end of string" indicator
        while(USART0_putchar(*s) == -1);
        s++;
    }
}



void USART0_Gets (char *s)
{
  unsigned char dataHere = 1;
  signed int tmp;
  char * ptr;

  ptr = s;
  while(dataHere)
  {
    tmp = USART0_getchar();
    if (tmp < 0)  //no data is available
    {
      dataHere = 0;
      *ptr = 0;    //string termination
    }
    else
    {
      *ptr = tmp;
      ptr++;
    }
  }
}


void USART0_Readline (char *s)
{
  unsigned char lineComplete = 0;
  signed int tmp;
  char * ptr;

  ptr = s;
  while (lineComplete != 1)
  {
    tmp = USART0_getchar();
    if (tmp < 0)  //no data is available
    {}        //do nothing (wait)
    else      //character received
    {
      *ptr = (char) tmp;
      if ((((char) tmp) == '\n') || (((char) tmp)  == '\r') || (((char) tmp)  == 0) || ((ptr - s) > (180 - 2))) //abort after 180 chars
      {
        ptr++;
        *ptr = 0;  //terminate string
        lineComplete = 1;
      }
      ptr++;
    }
  }
}

void write_hex(uint8_t hexval)
{
	USART0_putchar(hex[hexval>>4]);
	USART0_putchar(hex[hexval&0x0f]);
	USART0_putchar(32);
}

void write_nl(void)
{
	USART0_putchar(10);
	USART0_putchar(13);
    USART0_putchar(10);
	USART0_putchar(13);
}
//---------------------------------------------------------------------
//
//---------------------------------------------------------------------

